#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include "util.h"
#include "config.h"

static char *getxmlattr(xmlAttr *a, char *name, char *err)
{
	for (; a && strcmp(a->name, name); a = a->next);
	if (!a)
		return err;
	return a->children->content;
}

char *get_xmpchapters(char *xmp)
{
	int i;
	char *ptr;
	size_t size;
	FILE *stream;
	xmlDocPtr doc;
	xmlXPathContextPtr xpath;
	xmlXPathObjectPtr res;
	xmlNodePtr node;
	double framerate, start;
	if (!(doc = xmlParseMemory(xmp, strlen(xmp))))
		return "";
	if (!(xpath = xmlXPathNewContext(doc)))
		return "";
	xmlXPathRegisterNs(xpath, "x", "adobe:ns:meta/");
	xmlXPathRegisterNs(xpath, "rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
	xmlXPathRegisterNs(xpath, "xmpDM", "http://ns.adobe.com/xmp/1.0/DynamicMedia/");
	res = xmlXPathEvalExpression("/x:xmpmeta/rdf:RDF/rdf:Description/@xmpDM:videoFrameRate", xpath);
	if (!res || !res->nodesetval->nodeNr)
		return "";
	framerate = strtod(res->nodesetval->nodeTab[0]->children->content, 0);
	xmlXPathFreeObject(res);
	res = xmlXPathEvalExpression("/x:xmpmeta/rdf:RDF/rdf:Description/xmpDM:Tracks/rdf:Bag/rdf:li/rdf:Description/xmpDM:markers/rdf:Seq/rdf:li/rdf:Description", xpath);
	if (!res || !res->nodesetval)
		return "";
	ptr = 0;
	size = 0;
	if (!(stream = open_memstream(&ptr, &size)))
		exit(99);
	fprintf(stream, "[");
	for (i = 0; i < res->nodesetval->nodeNr; i ++)
	{
		start = atoi(getxmlattr(res->nodesetval->nodeTab[i]->properties, "startTime", "0"))/framerate;
		if (i)
			fprintf(stream, ",");
		fprintf(stream, "{\"time\": %f, \"text\": \"%s\"}", start,
				jescape(getxmlattr(res->nodesetval->nodeTab[i]->properties, "name", "")));
	}
	fprintf(stream, "]");
	fclose(stream);
	return ptr;
}

int main(int argc, char *argv[])
{
	int err;
	char *path, *xmp;
	AVFormatContext *demux, *mux;
	AVDictionaryEntry *tag;
	AVDictionary *opts;
	if (argc != 5)
		return 1;
	av_register_all();
	init_avlogbuf();

	jobid = atoi(argv[1]);
	path = mprintf("%s/%s", CONFIG_VIDEOS_RAW, jstr(jlookup(argv[4], "path"), ""));
	ping_job(jobid, "running", 0);

	opts = 0;
	av_dict_set_int(&opts, "export_xmp", 1, 0);
	demux = 0;
	if (err = avformat_open_input(&demux, path, 0, &opts))
		job_failed("Opening input file failed: %s", av_err2str(err));
	avformat_find_stream_info(demux, 0);
	if (tag = av_dict_get(demux->metadata, "xmp", 0, 0))
		ping_job(jobid, "finished", "{\"xmp_chapters\": %s, \"duration\": %f, \"log\": \"%s\"}",
				get_xmpchapters(tag->value), demux->duration*av_q2d(AV_TIME_BASE_Q),
				jescape(get_avlogbuf()));
	else
		ping_job(jobid, "finished", "{\"duration\": %f, \"log\": \"%s\"}",
				demux->duration*av_q2d(AV_TIME_BASE_Q), jescape(get_avlogbuf()));
	return 0;
}
