#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>

#include "util.h"
#include "config.h"

int main(int argc, char *argv[])
{
	int i, err;
	int *idxmap;
	char *p, *path, *tmp;
	AVFormatContext *demux, *mux;
	AVPacket pkt;
	AVStream *stream;
	AVDictionary *muxopts;
	if (argc != 5)
		return 1;
	av_register_all();
	init_avlogbuf();
	memset(&pkt, 0, sizeof(pkt));
	av_init_packet(&pkt);

	jobid = atoi(argv[1]);
	path = mprintf("%s/%s", CONFIG_VIDEOS_RELEASED, jstr(jlookup(argv[4], "path"), ""));
	tmp = mprintf("%s/.tmp-%i", CONFIG_VIDEOS_TMP, jobid);
	ping_job(jobid, "running", 0);

	demux = 0;
	if (err = avformat_open_input(&demux, path, 0, 0))
		job_failed("Opening input file failed: %s", av_err2str(err));
	avformat_find_stream_info(demux, 0);
	avformat_alloc_output_context2(&mux, 0, jstr(jlookup(argv[4], "format"), 0), path);
	av_dict_copy(&mux->metadata, demux->metadata, 0);
	parse_dict(&mux->metadata, jlookup(argv[4], "metadata"));
	parse_chapters(mux, jlookup(argv[4], "chapters"), demux->duration);
	idxmap = malloc(sizeof(int)*demux->nb_streams);
	for (i = 0; i < demux->nb_streams; i ++)
	{
		idxmap[i] = -1;
		if (demux->streams[i]->codecpar->codec_type != AVMEDIA_TYPE_AUDIO
				&& demux->streams[i]->codecpar->codec_type != AVMEDIA_TYPE_VIDEO
				&& demux->streams[i]->codecpar->codec_type != AVMEDIA_TYPE_SUBTITLE)
			continue;
		stream = avformat_new_stream(mux, 0);
		av_dict_copy(&stream->metadata, mux->streams[i]->metadata, 0);
		idxmap[i] = stream->index;
		avcodec_parameters_copy(mux->streams[i]->codecpar,
				demux->streams[i]->codecpar);
		mux->streams[i]->codecpar->codec_tag = 0;
		mux->streams[i]->time_base = demux->streams[i]->time_base;
	}
	avio_open(&mux->pb, tmp, AVIO_FLAG_WRITE);
	muxopts = 0;
	parse_dict(&muxopts, jlookup(argv[4], "options"));
	if ((err = avformat_write_header(mux, &muxopts)) < 0)
		job_failed("Writing temporary file failed: %s", av_err2str(err));
	while (!av_read_frame(demux, &pkt))
	{
		if (pkt.stream_index >= demux->nb_streams
				|| idxmap[pkt.stream_index] == -1)
			continue;
		pkt.stream_index = idxmap[pkt.stream_index];
		pkt.pts = av_rescale_q_rnd(pkt.pts, demux->streams[pkt.stream_index]->time_base,
				mux->streams[pkt.stream_index]->time_base, AV_ROUND_NEAR_INF|AV_ROUND_PASS_MINMAX);
		pkt.dts = av_rescale_q_rnd(pkt.dts, demux->streams[pkt.stream_index]->time_base,
				mux->streams[pkt.stream_index]->time_base, AV_ROUND_NEAR_INF|AV_ROUND_PASS_MINMAX);
		pkt.duration = av_rescale_q(pkt.duration, demux->streams[pkt.stream_index]->time_base,
				mux->streams[pkt.stream_index]->time_base);
		pkt.pos = -1;
		if (err = av_interleaved_write_frame(mux, &pkt))
			job_failed("Could not write frame: %s", av_err2str(err));
	}
	av_interleaved_write_frame(mux, 0);
	if (err = av_write_trailer(mux))
		job_failed("Error writing trailer to temporary file", av_err2str(err));
	avio_closep(&mux->pb);
	if (rename(tmp, path))
		job_failed("Overwriting output file failed: %s", strerror(errno));
	unlink(tmp);
	ping_job(jobid, "finished", "{\"log\": \"%s\"}", jescape(get_avlogbuf()));
	return 0;
}
