#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "../util.h"

static char *skip_ws(char *s)
{
	for (; *s == ' ' || *s == '\t' || *s == '\n' || *s == '\r'; s ++);
	return s;
}

static char *skip(char *s)
{
	s = skip_ws(s);
	if (*s == '"')
	{
		for (s++; *s && *s != '"'; s ++)
			if (s[0] == '\\' && s[1] == '"')
				s += 1;
		if (*s == '"')
			s ++;
	}
	else if (*s == '-' || (*s >= '1' && *s <= '9'))
		for (s ++; (*s >= '0' && *s <= '9') || *s == 'e' || *s == 'E' || *s == '.' ||
				*s == '-' || *s == '+'; s ++);
	else if (*s == '[')
	{
		do
			s = skip_ws(skip(s+1));
		while (*s == ',');
		if (*s == ']')
			s ++;
	}
	else if (*s == '{')
	{
		do
		{
			s = skip_ws(skip(s+1));
			if (*s == ':')
				s ++;
			s = skip_ws(skip(s));
		}
		while (*s == ',');
		if (*s == '}')
			s ++;
	}
	else if (!strncmp(s, SL("true")))
		return s+4;
	else if (!strncmp(s, SL("false")))
		return s+5;
	else if (!strncmp(s, SL("null")))
		return s+4;
	return s;
}

size_t parse_escape(char *s, uint32_t *code)
{
	char buf[4] = {};
	if (*s == 'u')
	{
		if (!(buf[0] = s[1]) ||
				!(buf[1] = s[2]) ||
				!(buf[2] = s[3]) ||
				!(buf[3] = s[4]))
			return 0;
		*code = strtol(buf, 0, 16);
		return 5;
	}
	switch (*s)
	{
		case 'b': *code = '\b'; break;
		case 'f': *code = '\f'; break;
		case 'n': *code = '\n'; break;
		case 'r': *code = '\r'; break;
		case 't': *code = '\t'; break;
		default:
			*code = *s;
	}
	return 1;
}

#define utf8_len(code) utf8_enc(code, 0)
size_t utf8_enc(uint32_t code, char *buf)
{
	char _buf[4];
	if (!buf)
		buf = _buf;
	if (code <= 0x7f)
	{
		buf[0] = code;
		return 1;
	}
	else if (code <= 0x7ff)
	{
		buf[0] = 0xc0 | (code >> 6);
		buf[1] = 0x80 | (code & 0x3f);
		return 2;
	}
	else if (code <= 0xffff)
	{
		buf[0] = 0xe0 | (code >> 12);
		buf[1] = 0x80 | ((code >> 6) & 0x3f);
		buf[2] = 0x80 | (code & 0x3f);
		return 3;
	}
	else
		return 0;
}

ssize_t jbin(char *s, char *buf, size_t len)
{
	int i, tmp;
	uint32_t code;
	if (!s)
		return -1;
	s = skip_ws(s);
	if (*s != '"')
		return -1;
	for (i = 0, s ++; *s && *s != '"' && i+1 < len;)
		if (*s == '\\')
		{
			if (!(tmp = parse_escape(++s, &code)))
				return -1;
			s += tmp;
			if (i+utf8_len(code) >= len)
				return -1;
			i += utf8_enc(code, buf+i);
		}
		else
			buf[i ++] = *(s ++);
	if (*s != '"')
		return -1;
	return i;
}

char *jstrb(char *s, char *err, char *buf, size_t len)
{
	ssize_t res;
	res = jbin(s, buf, len);
	if (res == -1 || res+1 >= len)
		return err;
	buf[res] = 0;
	return buf;
}

int jint(char *s, int err)
{
	if (!s)
		return err;
	if (!strncmp(s, SL("true")))
		return 1;
	if (!strncmp(s, SL("false")))
		return 0;
	if (!strncmp(s, SL("null")))
		return 0;
	return strtol(s, 0, 10);
}

char *jenter(char *s)
{
	if (!s)
		return 0;
	s = skip_ws(s);
	if (*s == '[' || *s == '{')
		return s+1;
	return 0;
}

char *jnext(char *s)
{
	if (!s)
		return 0;
	s = skip_ws(skip(s));
	if (*s == ':')
		s = skip_ws(skip(s+1));
	if (*s == ',')
		return s+1;
	else
		return 0;
}

char *jvalue(char *s)
{
	if (!s)
		return 0;
	s = skip_ws(skip(s));
	if (*s != ':')
		return 0;
	return s+1;
}

char *jlookup(char *s, char *key)
{
	static __thread char buf[JSON_KEY_SIZE];
	if (!s)
		return 0;
	s = skip_ws(s);
	if (*s != '{')
		return 0;
	for (s = skip_ws(s+1); s; s = jnext(s))
		if (!strcmp(jstrb(s, "INVALID KEY", BL(buf)), key))
			return jvalue(s);
	return 0;
}
